@@ -2,10 +2,11 @@ import enum
 import pathlib
 import tempfile
 import warnings
-from typing import Any, Callable, Dict, List, Optional, Union, overload
+from typing import Any, Callable, Optional, Union, overload
 
 import pandas as pd
 
+from snowflake import snowpark
 from snowflake.ml._internal import telemetry
 from snowflake.ml._internal.utils import sql_identifier
 from snowflake.ml.lineage import lineage_node
@@ -14,7 +15,7 @@ from snowflake.ml.model._client.ops import metadata_ops, model_ops, service_ops
 from snowflake.ml.model._model_composer import model_composer
 from snowflake.ml.model._model_composer.model_manifest import model_manifest_schema
 from snowflake.ml.model._packager.model_handlers import snowmlmodel
-from snowflake.snowpark import Session, dataframe
+from snowflake.snowpark import Session, async_job, dataframe
 
 _TELEMETRY_PROJECT = "MLOps"
 _TELEMETRY_SUBPROJECT = "ModelManagement"
@@ -32,7 +33,7 @@ class ModelVersion(lineage_node.LineageNode):
     _service_ops: service_ops.ServiceOperator
     _model_name: sql_identifier.SqlIdentifier
     _version_name: sql_identifier.SqlIdentifier
-    _functions: List[model_manifest_schema.ModelFunctionInfo]
+    _functions: list[model_manifest_schema.ModelFunctionInfo]
 
     def __init__(self) -> None:
         raise RuntimeError("ModelVersion's initializer is not meant to be used. Use `version` from model instead.")
@@ -152,7 +153,7 @@ class ModelVersion(lineage_node.LineageNode):
         project=_TELEMETRY_PROJECT,
         subproject=_TELEMETRY_SUBPROJECT,
     )
-    def show_metrics(self) -> Dict[str, Any]:
+    def show_metrics(self) -> dict[str, Any]:
         """Show all metrics logged with the model version.
 
         Returns:
@@ -293,7 +294,7 @@ class ModelVersion(lineage_node.LineageNode):
             statement_params=statement_params,
         )
 
-    def _get_functions(self) -> List[model_manifest_schema.ModelFunctionInfo]:
+    def _get_functions(self) -> list[model_manifest_schema.ModelFunctionInfo]:
         statement_params = telemetry.get_statement_params(
             project=_TELEMETRY_PROJECT,
             subproject=_TELEMETRY_SUBPROJECT,
@@ -310,12 +311,12 @@ class ModelVersion(lineage_node.LineageNode):
         project=_TELEMETRY_PROJECT,
         subproject=_TELEMETRY_SUBPROJECT,
     )
-    def get_model_objective(self) -> model_types.ModelObjective:
+    def get_model_task(self) -> model_types.Task:
         statement_params = telemetry.get_statement_params(
             project=_TELEMETRY_PROJECT,
             subproject=_TELEMETRY_SUBPROJECT,
         )
-        return self._model_ops.get_model_objective(
+        return self._model_ops.get_model_task(
             database_name=None,
             schema_name=None,
             model_name=self._model_name,
@@ -327,7 +328,7 @@ class ModelVersion(lineage_node.LineageNode):
         project=_TELEMETRY_PROJECT,
         subproject=_TELEMETRY_SUBPROJECT,
     )
-    def show_functions(self) -> List[model_manifest_schema.ModelFunctionInfo]:
+    def show_functions(self) -> list[model_manifest_schema.ModelFunctionInfo]:
         """Show all functions information in a model version that is callable.
 
         Returns:
@@ -405,11 +406,6 @@ class ModelVersion(lineage_node.LineageNode):
             strict_input_validation: Enable stricter validation for the input data. This will result value range based
                 type validation to make sure your input data won't overflow when providing to the model.
 
-        Raises:
-            ValueError: When no method with the corresponding name is available.
-            ValueError: When there are more than 1 target methods available in the model but no function name specified.
-            ValueError: When the partition column is not a valid Snowflake identifier.
-
         Returns:
             The prediction data. It would be the same type dataframe as your input.
         """
@@ -422,7 +418,40 @@ class ModelVersion(lineage_node.LineageNode):
             # Partition column must be a valid identifier
             partition_column = sql_identifier.SqlIdentifier(partition_column)
 
-        functions: List[model_manifest_schema.ModelFunctionInfo] = self._functions
+        target_function_info = self._get_function_info(function_name=function_name)
+
+        if service_name:
+            database_name_id, schema_name_id, service_name_id = sql_identifier.parse_fully_qualified_name(service_name)
+
+            return self._model_ops.invoke_method(
+                method_name=sql_identifier.SqlIdentifier(target_function_info["name"]),
+                signature=target_function_info["signature"],
+                X=X,
+                database_name=database_name_id,
+                schema_name=schema_name_id,
+                service_name=service_name_id,
+                strict_input_validation=strict_input_validation,
+                statement_params=statement_params,
+            )
+        else:
+            return self._model_ops.invoke_method(
+                method_name=sql_identifier.SqlIdentifier(target_function_info["name"]),
+                method_function_type=target_function_info["target_method_function_type"],
+                signature=target_function_info["signature"],
+                X=X,
+                database_name=None,
+                schema_name=None,
+                model_name=self._model_name,
+                version_name=self._version_name,
+                strict_input_validation=strict_input_validation,
+                partition_column=partition_column,
+                statement_params=statement_params,
+                is_partitioned=target_function_info["is_partitioned"],
+            )
+
+    def _get_function_info(self, function_name: Optional[str]) -> model_manifest_schema.ModelFunctionInfo:
+        functions: list[model_manifest_schema.ModelFunctionInfo] = self._functions
+
         if function_name:
             req_method_name = sql_identifier.SqlIdentifier(function_name).identifier()
             find_method: Callable[[model_manifest_schema.ModelFunctionInfo], bool] = (
@@ -445,32 +474,7 @@ class ModelVersion(lineage_node.LineageNode):
         else:
             target_function_info = functions[0]
 
-        if service_name:
-            return self._model_ops.invoke_method(
-                method_name=sql_identifier.SqlIdentifier(target_function_info["name"]),
-                signature=target_function_info["signature"],
-                X=X,
-                database_name=None,
-                schema_name=None,
-                service_name=sql_identifier.SqlIdentifier(service_name),
-                strict_input_validation=strict_input_validation,
-                statement_params=statement_params,
-            )
-        else:
-            return self._model_ops.invoke_method(
-                method_name=sql_identifier.SqlIdentifier(target_function_info["name"]),
-                method_function_type=target_function_info["target_method_function_type"],
-                signature=target_function_info["signature"],
-                X=X,
-                database_name=None,
-                schema_name=None,
-                model_name=self._model_name,
-                version_name=self._version_name,
-                strict_input_validation=strict_input_validation,
-                partition_column=partition_column,
-                statement_params=statement_params,
-                is_partitioned=target_function_info["is_partitioned"],
-            )
+        return target_function_info
 
     @telemetry.send_api_usage_telemetry(
         project=_TELEMETRY_PROJECT, subproject=_TELEMETRY_SUBPROJECT, func_params_to_log=["export_mode"]
@@ -613,6 +617,110 @@ class ModelVersion(lineage_node.LineageNode):
             version_name=sql_identifier.SqlIdentifier(version),
         )
 
+    @overload
+    def create_service(
+        self,
+        *,
+        service_name: str,
+        image_build_compute_pool: Optional[str] = None,
+        service_compute_pool: str,
+        image_repo: str,
+        ingress_enabled: bool = False,
+        max_instances: int = 1,
+        cpu_requests: Optional[str] = None,
+        memory_requests: Optional[str] = None,
+        gpu_requests: Optional[str] = None,
+        num_workers: Optional[int] = None,
+        max_batch_rows: Optional[int] = None,
+        force_rebuild: bool = False,
+        build_external_access_integration: Optional[str] = None,
+        block: bool = True,
+    ) -> Union[str, async_job.AsyncJob]:
+        """Create an inference service with the given spec.
+
+        Args:
+            service_name: The name of the service, can be fully qualified. If not fully qualified, the database or
+                schema of the model will be used.
+            image_build_compute_pool: The name of the compute pool used to build the model inference image. It uses
+                the service compute pool if None.
+            service_compute_pool: The name of the compute pool used to run the inference service.
+            image_repo: The name of the image repository, can be fully qualified. If not fully qualified, the database
+                or schema of the model will be used.
+            ingress_enabled: If true, creates an service endpoint associated with the service. User must have
+                BIND SERVICE ENDPOINT privilege on the account.
+            max_instances: The maximum number of inference service instances to run. The same value it set to
+                MIN_INSTANCES property of the service.
+            cpu_requests: The cpu limit for CPU based inference. Can be an integer, fractional or string values. If
+                None, we attempt to utilize all the vCPU of the node.
+            memory_requests: The memory limit with for CPU based inference. Can be an integer or a fractional value, but
+                requires a unit (GiB, MiB). If None, we attempt to utilize all the memory of the node.
+            gpu_requests: The gpu limit for GPU based inference. Can be integer, fractional or string values. Use CPU
+                if None.
+            num_workers: The number of workers to run the inference service for handling requests in parallel within an
+                instance of the service. By default, it is set to 2*vCPU+1 of the node for CPU based inference and 1 for
+                GPU based inference. For GPU based inference, please see best practices before playing with this value.
+            max_batch_rows: The maximum number of rows to batch for inference. Auto determined if None. Minimum 32.
+            force_rebuild: Whether to force a model inference image rebuild.
+            build_external_access_integration: (Deprecated) The external access integration for image build. This is
+                usually permitting access to conda & PyPI repositories.
+            block: A bool value indicating whether this function will wait until the service is available.
+                When it is ``False``, this function executes the underlying service creation asynchronously
+                and returns an :class:`AsyncJob`.
+        """
+        ...
+
+    @overload
+    def create_service(
+        self,
+        *,
+        service_name: str,
+        image_build_compute_pool: Optional[str] = None,
+        service_compute_pool: str,
+        image_repo: str,
+        ingress_enabled: bool = False,
+        max_instances: int = 1,
+        cpu_requests: Optional[str] = None,
+        memory_requests: Optional[str] = None,
+        gpu_requests: Optional[str] = None,
+        num_workers: Optional[int] = None,
+        max_batch_rows: Optional[int] = None,
+        force_rebuild: bool = False,
+        build_external_access_integrations: Optional[list[str]] = None,
+        block: bool = True,
+    ) -> Union[str, async_job.AsyncJob]:
+        """Create an inference service with the given spec.
+
+        Args:
+            service_name: The name of the service, can be fully qualified. If not fully qualified, the database or
+                schema of the model will be used.
+            image_build_compute_pool: The name of the compute pool used to build the model inference image. It uses
+                the service compute pool if None.
+            service_compute_pool: The name of the compute pool used to run the inference service.
+            image_repo: The name of the image repository, can be fully qualified. If not fully qualified, the database
+                or schema of the model will be used.
+            ingress_enabled: If true, creates an service endpoint associated with the service. User must have
+                BIND SERVICE ENDPOINT privilege on the account.
+            max_instances: The maximum number of inference service instances to run. The same value it set to
+                MIN_INSTANCES property of the service.
+            cpu_requests: The cpu limit for CPU based inference. Can be an integer, fractional or string values. If
+                None, we attempt to utilize all the vCPU of the node.
+            memory_requests: The memory limit with for CPU based inference. Can be an integer or a fractional value, but
+                requires a unit (GiB, MiB). If None, we attempt to utilize all the memory of the node.
+            gpu_requests: The gpu limit for GPU based inference. Can be integer, fractional or string values. Use CPU
+                if None.
+            num_workers: The number of workers to run the inference service for handling requests in parallel within an
+                instance of the service. By default, it is set to 2*vCPU+1 of the node for CPU based inference and 1 for
+                GPU based inference. For GPU based inference, please see best practices before playing with this value.
+            max_batch_rows: The maximum number of rows to batch for inference. Auto determined if None. Minimum 32.
+            force_rebuild: Whether to force a model inference image rebuild.
+            build_external_access_integrations: The external access integrations for image build. This is usually
+                permitting access to conda & PyPI repositories.
+            block: A bool value indicating whether this function will wait until the service is available.
+                When it is ``False``, this function executes the underlying service creation asynchronously
+                and returns an :class:`AsyncJob`.
+        """
+        ...
+
     @telemetry.send_api_usage_telemetry(
         project=_TELEMETRY_PROJECT,
         subproject=_TELEMETRY_SUBPROJECT,
@@ -625,6 +733,7 @@ class ModelVersion(lineage_node.LineageNode):
             "image_repo",
             "gpu_requests",
             "num_workers",
+            "max_batch_rows",
         ],
     )
     def create_service(
@@ -636,37 +745,74 @@ class ModelVersion(lineage_node.LineageNode):
         image_repo: str,
         ingress_enabled: bool = False,
         max_instances: int = 1,
-        gpu_requests: Optional[str] = None,
+        cpu_requests: Optional[str] = None,
+        memory_requests: Optional[str] = None,
+        gpu_requests: Optional[Union[str, int]] = None,
         num_workers: Optional[int] = None,
+        max_batch_rows: Optional[int] = None,
         force_rebuild: bool = False,
-        build_external_access_integration: str,
-    ) -> str:
+        build_external_access_integration: Optional[str] = None,
+        build_external_access_integrations: Optional[list[str]] = None,
+        block: bool = True,
+    ) -> Union[str, async_job.AsyncJob]:
         """Create an inference service with the given spec.
 
         Args:
             service_name: The name of the service, can be fully qualified. If not fully qualified, the database or
                 schema of the model will be used.
-            image_build_compute_pool: The name of the compute pool used to build the model inference image. Use
+            image_build_compute_pool: The name of the compute pool used to build the model inference image. It uses
                 the service compute pool if None.
             service_compute_pool: The name of the compute pool used to run the inference service.
             image_repo: The name of the image repository, can be fully qualified. If not fully qualified, the database
                 or schema of the model will be used.
-            ingress_enabled: Whether to enable ingress.
-            max_instances: The maximum number of inference service instances to run.
+            ingress_enabled: If true, creates an service endpoint associated with the service. User must have
+                BIND SERVICE ENDPOINT privilege on the account.
+            max_instances: The maximum number of inference service instances to run. The same value it set to
+                MIN_INSTANCES property of the service.
+            cpu_requests: The cpu limit for CPU based inference. Can be an integer, fractional or string values. If
+                None, we attempt to utilize all the vCPU of the node.
+            memory_requests: The memory limit with for CPU based inference. Can be an integer or a fractional value, but
+                requires a unit (GiB, MiB). If None, we attempt to utilize all the memory of the node.
             gpu_requests: The gpu limit for GPU based inference. Can be integer, fractional or string values. Use CPU
                 if None.
-            num_workers: The number of workers (replicas of models) to run the inference service.
-                Auto determined if None.
+            num_workers: The number of workers to run the inference service for handling requests in parallel within an
+                instance of the service. By default, it is set to 2*vCPU+1 of the node for CPU based inference and 1 for
+                GPU based inference. For GPU based inference, please see best practices before playing with this value.
+            max_batch_rows: The maximum number of rows to batch for inference. Auto determined if None. Minimum 32.
             force_rebuild: Whether to force a model inference image rebuild.
-            build_external_access_integration: The external access integration for image build.
+            build_external_access_integration: (Deprecated) The external access integration for image build. This is
+                usually permitting access to conda & PyPI repositories.
+            build_external_access_integrations: The external access integrations for image build. This is usually
+                permitting access to conda & PyPI repositories.
+            block: A bool value indicating whether this function will wait until the service is available.
+                When it is False, this function executes the underlying service creation asynchronously
+                and returns an AsyncJob.
+
+        Raises:
+            ValueError: Illegal external access integration arguments.
 
         Returns:
-            The service name.
+            If `block=True`, return result information about service creation from server.
+            Otherwise, return the service creation AsyncJob.
         """
         statement_params = telemetry.get_statement_params(
             project=_TELEMETRY_PROJECT,
             subproject=_TELEMETRY_SUBPROJECT,
         )
+        if build_external_access_integration is not None:
+            msg = (
+                "`build_external_access_integration` is deprecated. "
+                "Please use `build_external_access_integrations` instead."
+            )
+            warnings.warn(msg, DeprecationWarning, stacklevel=2)
+            if build_external_access_integrations is not None:
+                msg = (
+                    "`build_external_access_integration` and `build_external_access_integrations` cannot be set at the"
+                    "same time. Please use `build_external_access_integrations` only."
+                )
+                raise ValueError(msg)
+            build_external_access_integrations = [build_external_access_integration]
+
         service_db_id, service_schema_id, service_id = sql_identifier.parse_fully_qualified_name(service_name)
         image_repo_db_id, image_repo_schema_id, image_repo_id = sql_identifier.parse_fully_qualified_name(image_repo)
         return self._service_ops.create_service(
@@ -688,10 +834,151 @@ class ModelVersion(lineage_node.LineageNode):
             image_repo_name=image_repo_id,
             ingress_enabled=ingress_enabled,
             max_instances=max_instances,
+            cpu_requests=cpu_requests,
+            memory_requests=memory_requests,
             gpu_requests=gpu_requests,
             num_workers=num_workers,
+            max_batch_rows=max_batch_rows,
             force_rebuild=force_rebuild,
-            build_external_access_integration=sql_identifier.SqlIdentifier(build_external_access_integration),
+            build_external_access_integrations=(
+                None
+                if build_external_access_integrations is None
+                else [sql_identifier.SqlIdentifier(eai) for eai in build_external_access_integrations]
+            ),
+            block=block,
+            statement_params=statement_params,
+        )
+
+    @telemetry.send_api_usage_telemetry(
+        project=_TELEMETRY_PROJECT,
+        subproject=_TELEMETRY_SUBPROJECT,
+    )
+    def list_services(
+        self,
+    ) -> pd.DataFrame:
+        """List all the service names using this model version.
+
+        Returns:
+            List of service_names: The name of the service, can be fully qualified. If not fully qualified, the database
+                or schema of the model will be used.
+        """
+        statement_params = telemetry.get_statement_params(
+            project=_TELEMETRY_PROJECT,
+            subproject=_TELEMETRY_SUBPROJECT,
+        )
+
+        return pd.DataFrame(
+            self._model_ops.show_services(
+                database_name=None,
+                schema_name=None,
+                model_name=self._model_name,
+                version_name=self._version_name,
+                statement_params=statement_params,
+            )
+        )
+
+    @telemetry.send_api_usage_telemetry(
+        project=_TELEMETRY_PROJECT,
+        subproject=_TELEMETRY_SUBPROJECT,
+    )
+    def delete_service(
+        self,
+        service_name: str,
+    ) -> None:
+        """Drops the given service.
+
+        Args:
+            service_name: The name of the service, can be fully qualified. If not fully qualified, the database or
+                schema of the model will be used.
+
+        Raises:
+            ValueError: If the service does not exist or operation is not permitted by user or service does not belong
+                to this model.
+        """
+        if not service_name:
+            raise ValueError("service_name cannot be empty.")
+
+        statement_params = telemetry.get_statement_params(
+            project=_TELEMETRY_PROJECT,
+            subproject=_TELEMETRY_SUBPROJECT,
+        )
+
+        database_name_id, schema_name_id, service_name_id = sql_identifier.parse_fully_qualified_name(service_name)
+        self._model_ops.delete_service(
+            database_name=None,
+            schema_name=None,
+            model_name=self._model_name,
+            version_name=self._version_name,
+            service_database_name=database_name_id,
+            service_schema_name=schema_name_id,
+            service_name=service_name_id,
+            statement_params=statement_params,
+        )
+
+    @snowpark._internal.utils.private_preview(version="1.8.3")
+    @telemetry.send_api_usage_telemetry(
+        project=_TELEMETRY_PROJECT,
+        subproject=_TELEMETRY_SUBPROJECT,
+    )
+    def _run_job(
+        self,
+        X: Union[pd.DataFrame, "dataframe.DataFrame"],
+        *,
+        job_name: str,
+        compute_pool: str,
+        image_repo: str,
+        output_table_name: str,
+        function_name: Optional[str] = None,
+        cpu_requests: Optional[str] = None,
+        memory_requests: Optional[str] = None,
+        gpu_requests: Optional[Union[str, int]] = None,
+        num_workers: Optional[int] = None,
+        max_batch_rows: Optional[int] = None,
+        force_rebuild: bool = False,
+        build_external_access_integrations: Optional[list[str]] = None,
+    ) -> Union[pd.DataFrame, dataframe.DataFrame]:
+        statement_params = telemetry.get_statement_params(
+            project=_TELEMETRY_PROJECT,
+            subproject=_TELEMETRY_SUBPROJECT,
+        )
+        target_function_info = self._get_function_info(function_name=function_name)
+        job_db_id, job_schema_id, job_id = sql_identifier.parse_fully_qualified_name(job_name)
+        image_repo_db_id, image_repo_schema_id, image_repo_id = sql_identifier.parse_fully_qualified_name(image_repo)
+        output_table_db_id, output_table_schema_id, output_table_id = sql_identifier.parse_fully_qualified_name(
+            output_table_name
+        )
+        warehouse = self._service_ops._session.get_current_warehouse()
+        assert warehouse, "No active warehouse selected in the current session."
+        return self._service_ops.invoke_job_method(
+            target_method=target_function_info["target_method"],
+            signature=target_function_info["signature"],
+            X=X,
+            database_name=None,
+            schema_name=None,
+            model_name=self._model_name,
+            version_name=self._version_name,
+            job_database_name=job_db_id,
+            job_schema_name=job_schema_id,
+            job_name=job_id,
+            compute_pool_name=sql_identifier.SqlIdentifier(compute_pool),
+            warehouse_name=sql_identifier.SqlIdentifier(warehouse),
+            image_repo_database_name=image_repo_db_id,
+            image_repo_schema_name=image_repo_schema_id,
+            image_repo_name=image_repo_id,
+            output_table_database_name=output_table_db_id,
+            output_table_schema_name=output_table_schema_id,
+            output_table_name=output_table_id,
+            cpu_requests=cpu_requests,
+            memory_requests=memory_requests,
+            gpu_requests=gpu_requests,
+            num_workers=num_workers,
+            max_batch_rows=max_batch_rows,
+            force_rebuild=force_rebuild,
+            build_external_access_integrations=(
+                None
+                if build_external_access_integrations is None
+                else [sql_identifier.SqlIdentifier(eai) for eai in build_external_access_integrations]
+            ),
             statement_params=statement_params,
         )
 
