@@ -1,8 +1,9 @@
+import importlib
 import json
 import os
 import pathlib
 import warnings
-from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, cast
+from typing import Any, Callable, Iterable, Optional, Sequence, cast
 
 import numpy as np
 import numpy.typing as npt
@@ -10,8 +11,10 @@ import pandas as pd
 from absl import logging
 
 import snowflake.snowpark.dataframe as sp_df
+from snowflake.ml._internal import env
 from snowflake.ml._internal.utils import identifier
 from snowflake.ml.model import model_signature, type_hints as model_types
+from snowflake.ml.model._packager.model_env import model_env
 from snowflake.ml.model._packager.model_meta import model_meta
 from snowflake.ml.model._signatures import (
     core,
@@ -39,7 +42,7 @@ def _is_callable(model: model_types.SupportedModelType, method_name: str) -> boo
 
 
 def get_truncated_sample_data(
-    sample_input_data: model_types.SupportedDataType, length: int = 100
+    sample_input_data: model_types.SupportedDataType, length: int = 100, is_for_modeling_model: bool = False
 ) -> model_types.SupportedLocalDataType:
     trunc_sample_input = model_signature._truncate_data(sample_input_data, length=length)
     local_sample_input: model_types.SupportedLocalDataType = None
@@ -47,6 +50,8 @@ def get_truncated_sample_data(
         # Added because of Any from missing stubs.
         trunc_sample_input = cast(SnowparkDataFrame, trunc_sample_input)
         local_sample_input = snowpark_handler.SnowparkDataFrameHandler.convert_to_df(trunc_sample_input)
+        if is_for_modeling_model:
+            local_sample_input.columns = trunc_sample_input.columns
     else:
         local_sample_input = trunc_sample_input
     return local_sample_input
@@ -58,13 +63,15 @@ def validate_signature(
     target_methods: Iterable[str],
     sample_input_data: Optional[model_types.SupportedDataType],
     get_prediction_fn: Callable[[str, model_types.SupportedLocalDataType], model_types.SupportedLocalDataType],
+    is_for_modeling_model: bool = False,
 ) -> model_meta.ModelMetadata:
     if model_meta.signatures:
         validate_target_methods(model, list(model_meta.signatures.keys()))
         if sample_input_data is not None:
-            local_sample_input = get_truncated_sample_data(sample_input_data)
+            local_sample_input = get_truncated_sample_data(
+                sample_input_data, is_for_modeling_model=is_for_modeling_model
+            )
             for target_method in model_meta.signatures.keys():
-
                 model_signature_inst = model_meta.signatures.get(target_method)
                 if model_signature_inst is not None:
                     # strict validation the input signature
@@ -77,7 +84,7 @@ def validate_signature(
     assert (
         sample_input_data is not None
     ), "Model signature and sample input are None at the same time. This should not happen with local model."
-    local_sample_input = get_truncated_sample_data(sample_input_data)
+    local_sample_input = get_truncated_sample_data(sample_input_data, is_for_modeling_model=is_for_modeling_model)
     for target_method in target_methods:
         predictions_df = get_prediction_fn(target_method, local_sample_input)
         sig = model_signature.infer_signature(
@@ -102,6 +109,35 @@ def get_input_signature(
     return input_sig
 
 
+def add_inferred_explain_method_signature(
+    model_meta: model_meta.ModelMetadata,
+    explain_method: str,
+    target_method: str,
+    background_data: model_types.SupportedDataType,
+    explain_fn: Callable[[model_types.SupportedLocalDataType], model_types.SupportedLocalDataType],
+    output_feature_names: Optional[Sequence[str]] = None,
+) -> model_meta.ModelMetadata:
+    inputs = get_input_signature(model_meta, target_method)
+    if output_feature_names is None:  # If not provided, assume output feature names are the same as input feature names
+        output_feature_names = [spec.name for spec in inputs]
+
+    if model_meta.model_type == "snowml":
+        suffixed_output_names = [identifier.concat_names([name, "_explanation"]) for name in output_feature_names]
+    else:
+        suffixed_output_names = [f"{name}_explanation" for name in output_feature_names]
+
+    truncated_background_data = get_truncated_sample_data(background_data, 5)
+    sig = model_signature.infer_signature(
+        input_data=truncated_background_data,
+        output_data=explain_fn(truncated_background_data),
+        input_feature_names=[spec.name for spec in inputs],
+        output_feature_names=suffixed_output_names,
+    )
+
+    model_meta.signatures[explain_method] = sig
+    return model_meta
+
+
 def add_explain_method_signature(
     model_meta: model_meta.ModelMetadata,
     explain_method: str,
@@ -227,10 +263,11 @@ def validate_model_task(passed_model_task: model_types.Task, inferred_model_task
 
 
 def get_explain_target_method(
-    model_metadata: model_meta.ModelMetadata, target_methods_list: List[str]
+    model_metadata: model_meta.ModelMetadata, target_methods_list: list[str]
 ) -> Optional[str]:
-    for method in model_metadata.signatures.keys():
-        if method in target_methods_list:
+    """Returns the first target method that is found in the model metadata signatures."""
+    for method in target_methods_list:
+        if method in model_metadata.signatures.keys():
             return method
     return None
 
@@ -244,7 +281,7 @@ def save_transformers_config_with_auto_map(local_model_path: str) -> None:
                 config_dict = json.load(f)
 
             # a. get repository and class_path from configs
-            auto_map_configs = cast(Dict[str, str], config_dict.get("auto_map", {}))
+            auto_map_configs = cast(dict[str, str], config_dict.get("auto_map", {}))
             for config_name, config_value in auto_map_configs.items():
                 repository, _, class_path = config_value.rpartition("--")
 
@@ -257,3 +294,12 @@ def save_transformers_config_with_auto_map(local_model_path: str) -> None:
 
             with open(f_path, "w") as f:
                 json.dump(config_dict, f)
+
+
+def get_default_cuda_version() -> str:
+    # Default to the env cuda version when running in ML runtime
+    if env.IN_ML_RUNTIME and importlib.util.find_spec("torch") is not None:
+        import torch
+
+        return torch.version.cuda or model_env.DEFAULT_CUDA_VERSION
+    return model_env.DEFAULT_CUDA_VERSION
